@@ -72,7 +72,7 @@ GLOBAL void
 Client_Init( void )
 {
 	struct hostent *h;
-	
+
 	This_Server = New_Client_Struct( );
 	if( ! This_Server )
 	{
@@ -98,7 +98,7 @@ Client_Init( void )
 	Client_SetInfo( This_Server, Conf_ServerInfo );
 
 	My_Clients = This_Server;
-	
+
 	memset( &My_Whowas, 0, sizeof( My_Whowas ));
 } /* Client_Init */
 
@@ -111,7 +111,7 @@ Client_Exit( void )
 
 	if( NGIRCd_SignalRestart ) Client_Destroy( This_Server, "Server going down (restarting).", NULL, false );
 	else Client_Destroy( This_Server, "Server going down.", NULL, false );
-	
+
 	cnt = 0;
 	c = My_Clients;
 	while(c) {
@@ -228,7 +228,7 @@ GLOBAL void
 Client_Destroy( CLIENT *Client, const char *LogMsg, const char *FwdMsg, bool SendQuit )
 {
 	/* remove a client */
-	
+
 	CLIENT *last, *c;
 	char msg[COMMAND_LEN];
 	const char *txt;
@@ -337,7 +337,11 @@ Client_SetHostname( CLIENT *Client, const char *Hostname )
 	assert(Client != NULL);
 	assert(Hostname != NULL);
 
-	if (Conf_CloakHost[0]) {
+	/* Only cloak the hostmask if it has not yet been cloaked.
+	 * The period or colon indicates it's still an IP address.
+	 * An empty string means a rDNS lookup did not happen (yet). */
+	if (Conf_CloakHost[0] && (!Client->host[0] || strchr(Client->host, '.')
+				  || strchr(Client->host, ':'))) {
 		char cloak[GETID_LEN];
 
 		strlcpy(cloak, Hostname, GETID_LEN);
@@ -381,7 +385,7 @@ Client_SetID( CLIENT *Client, const char *ID )
 {
 	assert( Client != NULL );
 	assert( ID != NULL );
-	
+
 	strlcpy( Client->id, ID, sizeof( Client->id ));
 
 	if (Conf_CloakUserToNick) {
@@ -698,7 +702,7 @@ Client_ID( CLIENT *Client )
 	if(Client->type == CLIENT_USER)
 		assert(strlen(Client->id) < Conf_MaxNickLength);
 #endif
-						   
+
 	if( Client->id[0] ) return Client->id;
 	else return "*";
 } /* Client_ID */
