@@ -1,6 +1,6 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2014 Alexander Barton (alex@barton.de) and Contributors.
+ * Copyright (c)2001-2021 Alexander Barton (alex@barton.de) and Contributors.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,6 +9,7 @@
  * Please read the file COPYING, README and AUTHORS for more information.
  */
 
+#define GLOBAL_INIT
 #include "portab.h"
 
 /**
@@ -125,7 +126,7 @@ main(int argc, const char *argv[])
 			if (strcmp(argv[i], "--help") == 0) {
 				Show_Version();
 				puts(""); Show_Help( ); puts( "" );
-				exit(1);
+				exit(0);
 			}
 			if (strcmp(argv[i], "--nodaemon") == 0) {
 				NGIRCd_NoDaemon = true;
@@ -143,7 +144,7 @@ main(int argc, const char *argv[])
 #endif
 			if (strcmp(argv[i], "--version") == 0) {
 				Show_Version();
-				exit(1);
+				exit(0);
 			}
 		}
 		else if(argv[i][0] == '-' && argv[i][1] != '-') {
@@ -200,21 +201,23 @@ main(int argc, const char *argv[])
 				}
 
 				if (!ok) {
-					printf("%s: invalid option \"-%c\"!\n",
-					       PACKAGE_NAME, argv[i][n]);
-					printf("Try \"%s --help\" for more information.\n",
-					       PACKAGE_NAME);
-					exit(1);
+					fprintf(stderr,
+						"%s: invalid option \"-%c\"!\n",
+						PACKAGE_NAME, argv[i][n]);
+					fprintf(stderr,
+						"Try \"%s --help\" for more information.\n",
+						PACKAGE_NAME);
+					exit(2);
 				}
 			}
 
 		}
 		if (!ok) {
-			printf("%s: invalid option \"%s\"!\n",
-			       PACKAGE_NAME, argv[i]);
-			printf("Try \"%s --help\" for more information.\n",
-			       PACKAGE_NAME);
-			exit(1);
+			fprintf(stderr, "%s: invalid option \"%s\"!\n",
+				PACKAGE_NAME, argv[i]);
+			fprintf(stderr, "Try \"%s --help\" for more information.\n",
+				PACKAGE_NAME);
+			exit(2);
 		}
 	}
 
@@ -246,26 +249,18 @@ main(int argc, const char *argv[])
 		NGIRCd_SignalRestart = false;
 		NGIRCd_SignalQuit = false;
 
-		/* Initialize modules, part I */
 		Log_Init(!NGIRCd_NoDaemon);
 		Random_Init();
 		Conf_Init();
 		Log_ReInit();
 
-		/* Initialize the "main program": chroot environment, user and
-		 * group ID, ... */
+		/* Initialize the "main program":
+		 * chroot environment, user and group ID, ... */
 		if (!NGIRCd_Init(NGIRCd_NoDaemon)) {
 			Log(LOG_ALERT, "Fatal: Initialization failed, exiting!");
 			exit(1);
 		}
 
-		/* Initialize modules, part II: these functions are eventually
-		 * called with already dropped privileges ... */
-		Channel_Init();
-		Client_Init();
-		Conn_Init();
-		Class_Init();
-
 		if (!io_library_init(CONNECTION_POOL)) {
 			Log(LOG_ALERT,
 			    "Fatal: Could not initialize IO routines: %s",
@@ -280,6 +275,11 @@ main(int argc, const char *argv[])
 			exit(1);
 		}
 
+		Channel_Init();
+		Conn_Init();
+		Class_Init();
+		Client_Init();
+
 		/* Create protocol and server identification. The syntax
 		 * used by ngIRCd in PASS commands and the known "extended
 		 * flags" are described in doc/Protocol.txt. */
@@ -451,7 +451,7 @@ static void
 Show_Version( void )
 {
 	puts( NGIRCd_Version );
-	puts( "Copyright (c)2001-2014 Alexander Barton (<alex@barton.de>) and Contributors." );
+	puts( "Copyright (c)2001-2021 Alexander Barton (<alex@barton.de>) and Contributors." );
 	puts( "Homepage: <http://ngircd.barton.de/>\n" );
 	puts( "This is free software; see the source for copying conditions. There is NO" );
 	puts( "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." );
@@ -530,7 +530,7 @@ Pidfile_Create(pid_t pid)
 		close(pidfd);
 		return;
 	}
-	
+
 	if (write(pidfd, pidbuf, (size_t)len) != (ssize_t)len)
 		Log(LOG_ERR, "Can't write PID file (%s): %s!", Conf_PidFile,
 		    strerror(errno));
@@ -721,9 +721,10 @@ NGIRCd_Init(bool NGIRCd_NoDaemon)
 			Log(LOG_ERR, "Can't change group ID to %s(%u): %s!",
 			    grp ? grp->gr_name : "?", Conf_GID,
 			    strerror(real_errno));
-			if (real_errno != EPERM) 
+			if (real_errno != EPERM)
 				goto out;
 		}
+#ifdef HAVE_SETGROUPS
 		if (setgroups(0, NULL) != 0) {
 			real_errno = errno;
 			Log(LOG_ERR, "Can't drop supplementary group IDs: %s!",
@@ -731,6 +732,10 @@ NGIRCd_Init(bool NGIRCd_NoDaemon)
 			if (real_errno != EPERM)
 				goto out;
 		}
+#else
+		Log(LOG_WARNING,
+		    "Can't drop supplementary group IDs: setgroups(3) missing!");
+#endif
 	}
 #endif
 
